from clld.web.util import htmllib


def concepticon_link(request, meaning):
    if not meaning.concepticon_id:
        return ''
    return htmllib.HTML.a(
        htmllib.HTML.img(
            src=request.static_url('clld_cognacy_plugin:static/concepticon_logo.png'),
            height=20,
            width=30),
        title='corresponding concept set at Concepticon',
        href="http://concepticon.clld.org/parameters/{0}".format(meaning.concepticon_id))



# coding: utf8
from __future__ import unicode_literals, print_function, division

from clld.web.adapters.geojson import GeoJson
from clld import interfaces


class GeoJsonCognateset(GeoJson):

    """Render a parameter's values as geojson feature collection."""

    def featurecollection_properties(self, ctx, req):
        marker = req.registry.getUtility(interfaces.IMapMarker)
        return {
            'name': getattr(ctx, 'name', 'Values'),
            'domain': [
                {'icon': marker(de, req), 'id': de.id, 'name': de.name}
                for de in getattr(ctx, 'domain', [])]}

    def feature_iterator(self, ctx, req):
        return [cognate.counterpart.valueset for cognate in ctx.cognates]

    def get_language(self, ctx, req, valueset):
        return valueset.language

    def feature_properties(self, ctx, req, valueset):
        values = [co.counterpart for co in ctx.cognates]
        return {
            'label': ', '.join(v.name for v in valueset.values if v in values and v.name)
            or self.get_language(ctx, req, valueset).name}



from __future__ import unicode_literals

from clld_cognacy_plugin.models import Cognateset
from clld_cognacy_plugin.interfaces import ICognateset
from clld_cognacy_plugin.datatables import Cognatesets
from clld_cognacy_plugin.maps import CognatesetMap
from clld_cognacy_plugin.adapters import GeoJsonCognateset
from clld_cognacy_plugin.interfaces import ICognateset


def includeme(config):
    config.add_static_view('clld-cognacy-plugin-static', 'clld_cognacy_plugin:static')
    config.registry.settings['mako.directories'].append(
        'clld_cognacy_plugin:templates')
    config.register_resource('cognateset', Cognateset, ICognateset, with_index=True)
    config.register_datatable('cognatesets', Cognatesets)
    config.register_map('cognateset', CognatesetMap)
    config.register_adapter(
        GeoJsonCognateset, ICognateset, name=GeoJsonCognateset.mimetype)



# coding: utf8
from __future__ import unicode_literals, print_function, division

from clld.web.maps import Map, Layer


class CognatesetMap(Map):
    def get_layers(self):
        yield Layer(
            self.ctx.id,
            self.ctx.name,
            self.req.resource_url(self.ctx, ext='geojson'))

    def get_default_options(self):
        return {
            'show_labels': True,
            'info_query': {'parameter': self.ctx.pk},
            'hash': True}



# coding: utf8
from __future__ import unicode_literals, print_function, division

from clld.db.models.common import Parameter
from clld.web.datatables.base import DataTable, LinkCol, IdCol, Col
from clld.web.datatables.parameter import Parameters

from clld_cognacy_plugin.util import concepticon_link


class Cognatesets(DataTable):
    def col_defs(self):
        return [
            IdCol(self, 'id'),
            LinkCol(self, 'name'),
        ]


class ConcepticonCol(Col):
    __kw__ = dict(bSearchable=False)

    def format(self, item):
        return concepticon_link(self.dt.req, item)


class Meanings(Parameters):
    def col_defs(self):
        meaning_cls = list(Parameter.__subclasses__())[0]
        return [
            LinkCol(self, 'name'),
            Col(self, 'description'),
            ConcepticonCol(self, '#', model_col=getattr(meaning_cls, 'concepticon_id')),
        ]



from __future__ import unicode_literals

from zope.interface import implementer
import sqlalchemy as sa
import sqlalchemy.orm  # noqa: F401

from clld.db.meta import Base, PolymorphicBaseMixin
from clld.db.models.common import (
    Contribution, Value,
    IdNameDescriptionMixin, HasDataMixin, HasFilesMixin, HasSourceMixin, DataMixin,
    FilesMixin,
)
from clld_cognacy_plugin.interfaces import ICognateset


class MeaningMixin(object):
    concepticon_id = sa.Column(sa.Integer)


class Cognateset_data(Base, DataMixin):
    pass


class Cognateset_files(Base, FilesMixin):
    pass


@implementer(ICognateset)
class Cognateset(Base,
                 PolymorphicBaseMixin,
                 IdNameDescriptionMixin,
                 HasDataMixin,
                 HasFilesMixin):
    contribution_pk = sa.Column(sa.Integer, sa.ForeignKey('contribution.pk'))
    contribution = sa.orm.relationship(Contribution, backref='cognatesets')


class Cognate(Base):
    """
    The association table between counterparts for concepts in particular languages and
    cognate sets.
    """
    cognateset_pk = sa.Column(sa.Integer, sa.ForeignKey('cognateset.pk'))
    cognateset = sa.orm.relationship(Cognateset, backref='cognates')
    counterpart_pk = sa.Column(sa.Integer, sa.ForeignKey('value.pk'))
    counterpart = sa.orm.relationship(Value, backref='cognates')
    doubt = sa.Column(sa.Boolean, default=False)
    alignment = sa.Column(sa.Unicode)


class CognateReference(Base, HasSourceMixin):
    cognate_pk = sa.Column(sa.Integer, sa.ForeignKey('cognate.pk'))
    cognate = sa.orm.relationship(Cognate, backref="references")


class CognatesetReference(Base, HasSourceMixin):
    cognateset_pk = sa.Column(sa.Integer, sa.ForeignKey('cognateset.pk'))
    cognateset = sa.orm.relationship(Cognateset, backref="references")
